# -*- coding: utf-8 -*-

from django.db import models
from django.forms.extras.widgets import SelectDateWidget
from django.http import Http404
from django.shortcuts import get_object_or_404
from django.utils.translation import ugettext as _
import re


class Category(models.Model):
    title = models.CharField(max_length=250, 
                             help_text=_(u"Заглавие"))
    slug = models.SlugField(unique=True, 
                            help_text=_(u"Часть URL по которому будет доступна данная категория"))
    parent = models.ForeignKey('self', blank=True, null=True,
                               help_text=_(u"Верхняя категория"))
    order = models.IntegerField(blank=True, null=True,
                                help_text=(u"Порядок сортировки"))

    class Meta:
        verbose_name        = _("Category")
        verbose_name_plural = _("Categories")

    def __unicode__(self):
        return self.title

    @models.permalink
    def get_absolute_url(self):
        return('articles_category',
               (),
               {'category_path': self.tree_path()} )

    def tree_path(self):
        def p(c):
            yield c
            while c.parent:
                yield c.parent
                c = c.parent
        return('/'.join(reversed([i.slug for i in p(self)])))

    @classmethod
    def get_category(cls, category_path):
        categories_slugs = re.split(r'/+', category_path.strip('/'))
        if len(categories_slugs) <= 0:
            raise Http404
        for i, slug in enumerate(categories_slugs):
            if i == 0:
                c = get_object_or_404(cls, slug=slug)
            else:
                try:
                    c = c.category_set.get(slug=slug)
                except cls.DoesNotExist:
                    raise Http404
        return c


class Entry(models.Model):
    title = models.CharField(max_length=250, 
                             help_text=_(u"Заглавие"))
    slug = models.SlugField(unique=True, 
                            help_text=_(u"Часть URL по которому будет доступна данная категория"))
    excerpt = models.TextField(blank=True,
                               help_text=_(u"Краткое содержание"))
    body = models.TextField(help_text=_(u"Полный текст"))
    category = models.ForeignKey(Category,
                                 help_text=_(u"Категория"))
    date = models.DateField(blank=True, null=True, 
                            help_text=_(u"Дата публикации"))
    author = models.CharField(max_length=250, blank=True, null=True, 
                              help_text=_(u"Автор статьи или источник"))
    # in_navigation = models.BooleanField(default=False,
    #                                     help_text=_(u"Выводить статью в меню"))
    in_list = models.BooleanField(default=True,
                                  help_text=_(u"Выводить статью в списке по категории"))
    is_active = models.BooleanField(default=True,
                                    help_text=_(u"Статья доступна для просмотра"))
    order = models.IntegerField(blank=True, null=True,
                                help_text=(u"Порядок сортировки"))

    class Meta:
        verbose_name        = _(u"Entry")
        verbose_name_plural = _(u"Entries")

    def __unicode__(self):
        return self.title

    @models.permalink
    def get_absolute_url(self):
        from django.core.urlresolvers import reverse
        return('articles_entry', 
               (), 
               {'category_path': self.category.tree_path(),
                'slug': self.slug } )
